/***************************************************************************
 *   Copyright (C) 2007  Antony Martin <martin.antony@yahoo.fr>            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU General Public License           *
 *   as published by the Free Software Foundation; either version 2        *
 *   of the License, or (at your option) any later version.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA          *
 ***************************************************************************/

/* Cree le : 22 decembre 2006
   derniere modification le 10/01/2008 */

#ifndef SCEMEMORY_H
#define SCEMEMORY_H

#include <SCE/core/SCECTypes.h>

#ifdef __cplusplus
extern "C"
{
#endif

/**
 * \ingroup memory
 * @{
 */

/**
 * \brief Main malloc wrapper
 * \see SCE_Mem_Alloc()
 */
#define SCE_malloc(size) SCE_Mem_Alloc(__FILE__, __LINE__, size)
/**
 * \brief Main calloc wrapper
 * \see SCE_Mem_Calloc()
 */
#define SCE_calloc(size, nb) SCE_Mem_Calloc(__FILE__, __LINE__, size, nb)
/**
 * \brief Main realloc wrapper
 * \see SCE_Mem_Realloc()
 */
#define SCE_realloc(ptr, size) SCE_Mem_Realloc(__FILE__, __LINE__, ptr, size)
/**
 * \brief Main free wrapper
 * \see SCE_Mem_Free()
 */
#define SCE_free(ptr) SCE_Mem_Free(ptr)

/** @} */

void* SCE_Mem_Alloc (const char*, unsigned int, size_t);
void* SCE_Mem_Calloc (const char*, unsigned int, size_t, size_t);
void* SCE_Mem_Realloc (const char*, unsigned int, void*, size_t);
void SCE_Mem_Free (void*);

int SCE_Mem_IsValid (void*);

/* duplique des donnees dans une nouvelle zone memoire */
void* SCE_Mem_Dup (void*, size_t);

/* converti le type d'un groupe de donnees */
void SCE_Mem_Convert (int, void*, int, void*, size_t);
/* idem, mais alloue de la memoire pour la destination */
void* SCE_Mem_ConvertDup (int, int, void*, size_t);

/* liste et affiche toutes les allocation non liberees */
void SCE_Mem_List (void);

/*
size_t SCE_Mem_GetSize(void*);

const char* SCE_Mem_GetFile(void*);

unsigned int SCE_Mem_GetLine(void*);*/

#ifdef __cplusplus
} /* extern "C" */
#endif


#endif /* guard */
