/***************************************************************************
 *   Copyright (C) 2008  Antony Martin <martin(dot)antony(at)yahoo(dot)fr> *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU General Public License           *
 *   as published by the Free Software Foundation; either version 2        *
 *   of the License, or (at your option) any later version.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA          *
 ***************************************************************************/

/* Cree le : 31 janvier 2006
   derniere modification le 06/05/2008 */

#include <string.h>
#include <SCE/SCEMinimal.h>

#include <SCE/core/SCECSupport.h>

/**
 * \file SCECSupport.c
 * \copydoc extenstionsupport
 * \brief OpenGL extensions support managment
 * \file SCECSupport.h
 * \copydoc extenstionsupport
 * \brief OpenGL extensions support managment
 */

/**
 * \defgroup extenstionsupport OpenGL extensions support managment
 * \ingroup core
 * \brief OpenGL extensions support managment
 * 
 * This module is usefull to determine if an OpenGL extension is supported by
 * the running implementation.
 */

/** @{ */


static unsigned char caps[SCE_NUM_CAPS];


static void SCE_CCheckCaps (void)
{
    caps[SCE_TEX_MULTI] =
    glewIsSupported ("GL_ARB_multitexture");

    caps[SCE_TEX_NON_POWER_OF_TWO] =
    glewIsSupported ("GL_ARB_texture_non_power_of_two");

    caps[SCE_TEX_HW_GEN_MIPMAP] =
    glewIsSupported ("GL_SGIS_generate_mipmap");

    caps[SCE_TEX_DXT1] = caps[SCE_TEX_DXT3] = caps[SCE_TEX_DXT5] =
    glewIsSupported ("GL_ARB_texture_compression") &&
    glewIsSupported ("GL_EXT_texture_compression_s3tc");

    caps[SCE_TEX_S3TC] =
    glewIsSupported ("GL_EXT_texture_compression_s3tc");

    caps[SCE_TEX_3DC] = SCE_FALSE;

    caps[SCE_VBO] =
    glewIsSupported ("GL_ARB_vertex_buffer_object");

    caps[SCE_FBO] =
    glewIsSupported ("GL_EXT_framebuffer_object");

    caps[SCE_PBO] =
    glewIsSupported ("GL_EXT_pixel_buffer_object") ||
    glewIsSupported ("GL_ARB_pixel_buffer_object");

    caps[SCE_VERTEX_SHADER_GLSL] =
    glewIsSupported ("GL_ARB_vertex_shader") &&
    glewIsSupported ("GL_ARB_shader_objects") &&
    glewIsSupported ("GL_ARB_shading_language_100");

    caps[SCE_FRAGMENT_SHADER_GLSL] =
    glewIsSupported ("GL_ARB_fragment_shader") &&
    glewIsSupported ("GL_ARB_shader_objects") &&
    glewIsSupported ("GL_ARB_shading_language_100");

#if 0
    caps[SCE_CG_SHADERS] =
    glewIsSupported ("..."); /* NOTE: a completer... ou pas ? */
#endif
    caps[SCE_CG_SHADERS] = 0;

    caps[SCE_OCCLUSION_QUERY] =
    glewIsSupported ("GL_ARB_occlusion_query");

    caps[SCE_MRT] = glewIsSupported ("GL_ARB_draw_buffers");
}

/**
 * \internal
 */
int SCE_CSupportInit (void)
{
    /* recherche des extensions supportees */
    SCE_CCheckCaps ();
    return SCE_OK;
}

/**
 * \brief Checks if an OpenGL extension is supported by the running
 *        implementation
 * \param ext an OpenGL extension name
 * \returns SCE_TRUE if the OpenGL implementation supports the extension,
 *          SCE_FALSE otherwise
 * 
 * \see SCE_CHasCap()
 */
int SCE_CSupport (const char *ext)
{
    return (strstr ((const char*)glGetString (GL_EXTENSIONS), ext) ?
            SCE_TRUE : SCE_FALSE);
}

/**
 * \brief Wrapper for the GLEW's glewIsSupported() function
 * \param ext an OpenGL extension name
 * \returns SCE_TRUE if both GLEW and the running OpenGL implementation supports
 *          the extension, SCE_FALSE otherwise
 * 
 * \see SCE_CSupport()
 */
int SCE_CGlewSupport (const char *ext)
{
    return glewIsSupported (ext);
}

/**
 * \brief Fastly checks if a OpenGL's extension supported by the core engine is
 *        supported by the implementation
 * 
 * This function is designed to fastly checks if one of the SCE_CCap extensions
 * is supported by the running implementation. This is mostly used internally 
 * and does not support all OpenGL extensions. For a full test, use the
 * SCE_CSupport() function instead of this one.
 * 
 * \see SCE_CCap
 * \see SCE_CSupport()
 */
int SCE_CHasCap (SCE_CCap cap)
{
    return caps[cap];
}

/** @} */
